@@ -22,7 +22,6 @@
 #define __BUILDER_CONTEXT_H__
 
 #include <gio/gio.h>
-#include <libsoup/soup.h>
 #include <curl/curl.h>
 #include "builder-options.h"
 #include "builder-utils.h"
@@ -30,6 +29,15 @@
 
 G_BEGIN_DECLS
 
+/* Same as SOUP_HTTP_URI_FLAGS, means all possible flags for http uris */
+
+#if GLIB_CHECK_VERSION (2, 68, 0)
+#define CONTEXT_HTTP_URI_FLAGS (G_URI_FLAGS_HAS_PASSWORD | G_URI_FLAGS_ENCODED_PATH | G_URI_FLAGS_ENCODED_QUERY | G_URI_FLAGS_ENCODED_FRAGMENT | G_URI_FLAGS_SCHEME_NORMALIZE)
+#else
+/* GLib 2.66 didn't support scheme-based normalization */
+#define CONTEXT_HTTP_URI_FLAGS (G_URI_FLAGS_HAS_PASSWORD | G_URI_FLAGS_ENCODED_PATH | G_URI_FLAGS_ENCODED_QUERY | G_URI_FLAGS_ENCODED_FRAGMENT)
+#endif
+
 /* BuilderContext defined in builder-cache.h to fix include loop */
 
 #define BUILDER_TYPE_CONTEXT (builder_context_get_type ())
@@ -65,11 +73,11 @@ void            builder_context_set_sources_urls (BuilderContext *self,
 gboolean        builder_context_download_uri (BuilderContext *self,
                                               const char     *url,
                                               const char    **mirrors,
+                                              const char     *http_referer,
                                               GFile          *dest,
                                               const char     *checksums[BUILDER_CHECKSUMS_LEN],
                                               GChecksumType   checksums_type[BUILDER_CHECKSUMS_LEN],
                                               GError        **error);
-SoupSession *   builder_context_get_soup_session (BuilderContext *self);
 CURL *          builder_context_get_curl_session (BuilderContext *self);
 const char *    builder_context_get_arch (BuilderContext *self);
 void            builder_context_set_arch (BuilderContext *self,
@@ -77,6 +85,7 @@ void            builder_context_set_arch (BuilderContext *self,
 const char *    builder_context_get_default_branch (BuilderContext *self);
 void            builder_context_set_default_branch (BuilderContext *self,
                                                     const char     *default_branch);
+gint64          builder_context_get_source_date_epoch (BuilderContext *self);
 void            builder_context_set_source_date_epoch (BuilderContext *self,
                                                        gint64 source_date_epoch);
 const char *    builder_context_get_stop_at (BuilderContext *self);
