@@ -38,8 +38,7 @@
 #include <sys/ioctl.h>
 
 #include <glib.h>
-#include "libglnx/libglnx.h"
-#include <libsoup/soup.h>
+#include "libglnx.h"
 #include <gio/gunixoutputstream.h>
 #include <gio/gunixinputstream.h>
 
@@ -1139,31 +1138,6 @@ flatpak_allocate_tmpdir (int           tmpdir_dfd,
 }
 
 
-SoupSession *
-flatpak_create_soup_session (const char *user_agent)
-{
-  SoupSession *soup_session;
-  const char *http_proxy;
-
-  soup_session = soup_session_new_with_options (SOUP_SESSION_USER_AGENT, user_agent,
-                                                SOUP_SESSION_SSL_USE_SYSTEM_CA_FILE, TRUE,
-                                                SOUP_SESSION_USE_THREAD_CONTEXT, TRUE,
-                                                SOUP_SESSION_TIMEOUT, 60,
-                                                SOUP_SESSION_IDLE_TIMEOUT, 60,
-                                                NULL);
-  http_proxy = g_getenv ("http_proxy");
-  if (http_proxy)
-    {
-      g_autoptr(SoupURI) proxy_uri = soup_uri_new (http_proxy);
-      if (!proxy_uri)
-        g_warning ("Invalid proxy URI '%s'", http_proxy);
-      else
-        g_object_set (soup_session, SOUP_SESSION_PROXY_URI, proxy_uri, NULL);
-    }
-
-  return soup_session;
-}
-
 CURL *
 flatpak_create_curl_session (const char *user_agent)
 {
@@ -1199,6 +1173,7 @@ typedef enum {
 
 /* In numerical order of more privs */
 typedef enum {
+  FLATPAK_FILESYSTEM_MODE_NONE         = 0,
   FLATPAK_FILESYSTEM_MODE_READ_ONLY    = 1,
   FLATPAK_FILESYSTEM_MODE_READ_WRITE   = 2,
   FLATPAK_FILESYSTEM_MODE_CREATE       = 3,
@@ -1773,6 +1748,13 @@ parse_filesystem_flags (const char *filesystem, FlatpakFilesystemMode *mode)
       if (mode)
         *mode = FLATPAK_FILESYSTEM_MODE_CREATE;
     }
+  else if (g_str_equal (filesystem, "host:reset"))
+    {
+      filesystem = "host-reset";
+
+      if (mode)
+        *mode = FLATPAK_FILESYSTEM_MODE_NONE;
+    }
 
   return g_strndup (filesystem, len);
 }
@@ -1813,9 +1795,12 @@ static void
 flatpak_context_remove_filesystem (FlatpakContext *context,
                                    const char     *what)
 {
+  FlatpakFilesystemMode mode;
+  g_autofree char *fs = parse_filesystem_flags (what, &mode);
+
   g_hash_table_insert (context->filesystems,
-                       parse_filesystem_flags (what, NULL),
-                       NULL);
+                       g_steal_pointer (&fs),
+                       GINT_TO_POINTER (mode));
 }
 
 static gboolean
@@ -2225,11 +2210,19 @@ flatpak_context_to_args (FlatpakContext *context,
       g_ptr_array_add (args, g_strdup_printf ("--system-%s-name=%s", flatpak_policy_to_string (policy), name));
     }
 
+  if (g_hash_table_lookup_extended (context->filesystems, "host-reset", NULL, NULL))
+    {
+      g_ptr_array_add (args, g_strdup ("--nofilesystem=host:reset"));
+    }
+
   g_hash_table_iter_init (&iter, context->filesystems);
   while (g_hash_table_iter_next (&iter, &key, &value))
     {
       FlatpakFilesystemMode mode = GPOINTER_TO_INT (value);
 
+      if (g_str_equal (key, "host-reset"))
+        continue;
+
       if (mode == FLATPAK_FILESYSTEM_MODE_READ_ONLY)
         g_ptr_array_add (args, g_strdup_printf ("--filesystem=%s:ro", (char *)key));
       else if (mode == FLATPAK_FILESYSTEM_MODE_READ_WRITE)
