@@ -395,11 +395,9 @@ git_mirror_submodules (const char     *repo_location,
             return FALSE;
 
           lines = g_strsplit (g_strstrip (ls_tree), "\n", 0);
+          /* There can be path in the .gitmodules file that are not submodules. */
           if (g_strv_length (lines) != 1)
-            {
-              g_set_error (error, G_IO_ERROR, G_IO_ERROR_FAILED, "Not a gitlink tree: %s", path);
-              return FALSE;
-            }
+            continue;
 
           words = g_strsplit_set (lines[0], " \t", 4);
 
@@ -409,7 +407,7 @@ git_mirror_submodules (const char     *repo_location,
           g_debug ("mirror submodule %s at revision %s\n", absolute_url, words[2]);
           if (shallow)
             {
-              if (!builder_git_shallow_mirror_ref (absolute_url, destination_path, words[2], context, error))
+              if (!builder_git_shallow_mirror_ref (absolute_url, destination_path, flags, words[2], context, error))
                 return FALSE;
             }
           else
@@ -652,6 +650,7 @@ builder_git_mirror_repo (const char     *repo_location,
 gboolean
 builder_git_shallow_mirror_ref (const char     *repo_location,
                                 const char     *destination_path,
+                                FlatpakGitMirrorFlags flags,
                                 const char     *ref,
                                 BuilderContext *context,
                                 GError        **error)
@@ -712,10 +711,13 @@ builder_git_shallow_mirror_ref (const char     *repo_location,
   if (current_commit == NULL)
     return FALSE;
 
-  if (!git_mirror_submodules (repo_location, destination_path, TRUE,
-                              FLATPAK_GIT_MIRROR_FLAGS_MIRROR_SUBMODULES | FLATPAK_GIT_MIRROR_FLAGS_DISABLE_FSCK,
-                              mirror_dir, current_commit, context, error))
-    return FALSE;
+  if (flags & FLATPAK_GIT_MIRROR_FLAGS_MIRROR_SUBMODULES)
+    {
+      if (!git_mirror_submodules (repo_location, destination_path, TRUE,
+                                  flags | FLATPAK_GIT_MIRROR_FLAGS_DISABLE_FSCK,
+                                  mirror_dir, current_commit, context, error))
+        return FALSE;
+    }
 
   return TRUE;
 }
@@ -790,11 +792,9 @@ git_extract_submodule (const char     *repo_location,
             return FALSE;
 
           lines = g_strsplit (g_strstrip (ls_tree), "\n", 0);
+          /* There can be path in the .gitmodules file that are not submodules. */
           if (g_strv_length (lines) != 1)
-            {
-              g_set_error (error, G_IO_ERROR, G_IO_ERROR_FAILED, "Not a gitlink tree: %s", path);
-              return FALSE;
-            }
+            continue;
 
           words = g_strsplit_set (lines[0], " \t", 4);
 
@@ -810,7 +810,7 @@ git_extract_submodule (const char     *repo_location,
             return FALSE;
 
           if (!git (checkout_dir, NULL, 0, error,
-                    "submodule", "update", "--init", path, NULL))
+                    "-c", "protocol.file.allow=always", "submodule", "update", "--init", path, NULL))
             return FALSE;
 
           child_dir = g_file_resolve_relative_path (checkout_dir, path);
@@ -864,3 +864,33 @@ builder_git_checkout (const char     *repo_location,
 
   return TRUE;
 }
+
+char *
+builder_git_get_default_branch (const char *repo_location)
+{
+  g_auto(GStrv) parts = NULL;
+  g_autofree char *output = NULL;
+  g_autoptr(GError) error = NULL;
+
+  if (!git (NULL, &output, 0, &error,
+            "ls-remote", "--symref", repo_location, "HEAD", NULL))
+    return g_strdup ("master");
+
+  /* Example output:
+   * $ git ls-remote --symref http://gitlab.gnome.org/GNOME/evince.git HEAD
+   * warning: redirigiendo a https://gitlab.gnome.org/GNOME/evince.git/
+   * ref: refs/heads/main	HEAD
+   * 7a5ceb841874bef4b91282eee1025b0335a21795	HEAD
+   */
+  parts = g_strsplit (output, "\t", 1);
+  if (g_strv_length (parts) > 1 && g_str_has_prefix (parts[1], "HEAD") &&
+      g_strstr_len (parts[0], -1, "ref: "))
+    {
+      char *branch = strrchr (parts[0], '/');
+      if (branch != NULL)
+        return g_strdup (branch + 1);
+    }
+
+  g_debug ("Failed to auto-detect default branch from git output");
+  return g_strdup ("master");
+}
\ No newline at end of file
