@@ -47,6 +47,7 @@ struct BuilderSourceArchive
   char         *dest_filename;
   gboolean      git_init;
   char         *archive_type;
+  char         *http_referer;
 };
 
 typedef struct
@@ -69,6 +70,7 @@ enum {
   PROP_MIRROR_URLS,
   PROP_GIT_INIT,
   PROP_ARCHIVE_TYPE,
+  PROP_HTTP_REFERER,
   LAST_PROP
 };
 
@@ -83,6 +85,7 @@ typedef enum {
   TAR_LZMA,
   TAR_LZOP,
   TAR_XZ,
+  TAR_ZST,
   ZIP,
   SEVENZ,
 } BuilderArchiveType;
@@ -90,7 +93,7 @@ typedef enum {
 static gboolean
 is_tar (BuilderArchiveType type)
 {
-  return (type >= TAR) && (type <= TAR_XZ);
+  return (type >= TAR) && (type <= TAR_ZST);
 }
 
 static const char *
@@ -122,6 +125,9 @@ tar_decompress_flag (BuilderArchiveType type)
 
     case TAR_XZ:
       return "-J";
+
+    case TAR_ZST:
+      return "--zstd";
     }
 }
 
@@ -139,6 +145,7 @@ builder_source_archive_finalize (GObject *object)
   g_free (self->dest_filename);
   g_strfreev (self->mirror_urls);
   g_free (self->archive_type);
+  g_free (self->http_referer);
 
   G_OBJECT_CLASS (builder_source_archive_parent_class)->finalize (object);
 }
@@ -197,6 +204,10 @@ builder_source_archive_get_property (GObject    *object,
       g_value_set_string (value, self->archive_type);
       break;
 
+    case PROP_HTTP_REFERER:
+      g_value_set_string (value, self->http_referer);
+      break;
+
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
     }
@@ -226,11 +237,23 @@ builder_source_archive_set_property (GObject      *object,
     case PROP_MD5:
       g_free (self->md5);
       self->md5 = g_value_dup_string (value);
+      if (self->md5 != NULL && self->md5[0] != '\0')
+        {
+          g_printerr ("The \"md5\" source property is deprecated due to the weakness of MD5 hashes.\n");
+          g_printerr ("Use the \"sha256\" property for the more secure SHA256 hash.\n");
+        }
+
       break;
 
     case PROP_SHA1:
       g_free (self->sha1);
       self->sha1 = g_value_dup_string (value);
+      if (self->sha1 != NULL && self->sha1[0] != '\0')
+        {
+          g_printerr ("The \"sha1\" source property is deprecated due to the weakness of SHA1 hashes.\n");
+          g_printerr ("Use the \"sha256\" property for the more secure SHA256 hash.\n");
+        }
+
       break;
 
     case PROP_SHA256:
@@ -267,6 +290,11 @@ builder_source_archive_set_property (GObject      *object,
       self->archive_type = g_value_dup_string (value);
       break;
 
+    case PROP_HTTP_REFERER:
+      g_free (self->http_referer);
+      self->http_referer = g_value_dup_string (value);
+      break;
+
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
     }
@@ -285,11 +313,11 @@ builder_source_archive_validate (BuilderSource  *source,
   return TRUE;
 }
 
-static SoupURI *
+static GUri *
 get_uri (BuilderSourceArchive *self,
          GError              **error)
 {
-  SoupURI *uri;
+  GUri *uri;
 
   if (self->url == NULL)
     {
@@ -297,10 +325,9 @@ get_uri (BuilderSourceArchive *self,
       return NULL;
     }
 
-  uri = soup_uri_new (self->url);
+  uri = g_uri_parse (self->url, CONTEXT_HTTP_URI_FLAGS, error);
   if (uri == NULL)
     {
-      g_set_error (error, G_IO_ERROR, G_IO_ERROR_FAILED, "Invalid URL '%s'", self->url);
       return NULL;
     }
   return uri;
@@ -312,7 +339,7 @@ get_download_location (BuilderSourceArchive *self,
                        gboolean             *is_local,
                        GError              **error)
 {
-  g_autoptr(SoupURI) uri = NULL;
+  g_autoptr(GUri) uri = NULL;
   const char *path;
   g_autofree char *base_name = NULL;
   g_autoptr(GFile) file = NULL;
@@ -323,7 +350,7 @@ get_download_location (BuilderSourceArchive *self,
   if (uri == NULL)
     return FALSE;
 
-  path = soup_uri_get_path (uri);
+  path = g_uri_get_path (uri);
 
   if (self->dest_filename)
     base_name = g_strdup (self->dest_filename);
@@ -438,6 +465,7 @@ builder_source_archive_download (BuilderSource  *source,
   if (!builder_context_download_uri (context,
                                      self->url,
                                      (const char **)self->mirror_urls,
+                                     self->http_referer,
                                      file,
                                      checksums,
                                      checksums_type,
@@ -463,16 +491,14 @@ tar (GFile   *dir,
 }
 
 static gboolean
-unzip (GFile   *dir,
-       GError **error,
-       ...)
+unzip (GFile       *dir,
+       const char  *zip_path,
+       GError     **error)
 {
   gboolean res;
-  va_list ap;
+  const char *argv[] = { "unzip", "-q", zip_path, NULL };
 
-  va_start (ap, error);
-  res = flatpak_spawn (dir, NULL, 0, error, "unzip", ap);
-  va_end (ap);
+  res = flatpak_spawnv (dir, NULL, 0, error, argv, NULL);
 
   return res;
 }
@@ -547,12 +573,19 @@ get_type (GFile *archivefile)
       g_str_has_suffix (lower, ".txz"))
     return TAR_XZ;
 
+  if (g_str_has_suffix (lower, ".tar.zst") ||
+      g_str_has_suffix (lower, ".tzst"))
+    return TAR_ZST;
+
   if (g_str_has_suffix (lower, ".zip"))
     return ZIP;
 
   if (g_str_has_suffix (lower, ".rpm"))
     return RPM;
 
+  if (g_str_has_suffix (lower, ".7z"))
+    return SEVENZ;
+
   return UNKNOWN;
 }
 
@@ -683,6 +716,7 @@ get_type_from_prop (BuilderSourceArchive *self)
       { "tar-lzip", TAR_LZIP },
       { "tar-lzma", TAR_LZMA },
       { "tar-xz", TAR_XZ },
+      { "tar-zst", TAR_ZST },
       { "zip", ZIP },
       { "7z", SEVENZ },
   };
@@ -742,7 +776,7 @@ builder_source_archive_extract (BuilderSource  *source,
       if (zip_dest == NULL)
         return FALSE;
 
-      if (!unzip (zip_dest, error, archive_path, NULL))
+      if (!unzip (zip_dest, archive_path, error))
         return FALSE;
 
       if (self->strip_components > 0)
@@ -967,6 +1001,13 @@ builder_source_archive_class_init (BuilderSourceArchiveClass *klass)
                                                         "",
                                                         NULL,
                                                         G_PARAM_READWRITE));
+  g_object_class_install_property (object_class,
+                                   PROP_HTTP_REFERER,
+                                   g_param_spec_string ("referer",
+                                                        "",
+                                                        "",
+                                                        NULL,
+                                                        G_PARAM_READWRITE));
 }
 
 static void
