@@ -44,6 +44,7 @@ struct BuilderSourceFile
   char         *sha256;
   char         *sha512;
   char         *dest_filename;
+  char         *http_referer;
 };
 
 typedef struct
@@ -63,6 +64,7 @@ enum {
   PROP_SHA512,
   PROP_DEST_FILENAME,
   PROP_MIRROR_URLS,
+  PROP_HTTP_REFERER,
   LAST_PROP
 };
 
@@ -78,6 +80,7 @@ builder_source_file_finalize (GObject *object)
   g_free (self->sha256);
   g_free (self->sha512);
   g_free (self->dest_filename);
+  g_free (self->http_referer);
   g_strfreev (self->mirror_urls);
 
   G_OBJECT_CLASS (builder_source_file_parent_class)->finalize (object);
@@ -125,6 +128,10 @@ builder_source_file_get_property (GObject    *object,
       g_value_set_boxed (value, self->mirror_urls);
       break;
 
+    case PROP_HTTP_REFERER:
+      g_value_set_string (value, self->http_referer);
+      break;
+
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
     }
@@ -154,11 +161,23 @@ builder_source_file_set_property (GObject      *object,
     case PROP_MD5:
       g_free (self->md5);
       self->md5 = g_value_dup_string (value);
+      if (self->md5 != NULL && self->md5[0] != '\0')
+        {
+          g_printerr ("The \"md5\" source property is deprecated due to the weakness of MD5 hashes.\n");
+          g_printerr ("Use the \"sha256\" property for the more secure SHA256 hash.\n");
+        }
+
       break;
 
     case PROP_SHA1:
       g_free (self->sha1);
       self->sha1 = g_value_dup_string (value);
+      if (self->sha1 != NULL && self->sha1[0] != '\0')
+        {
+          g_printerr ("The \"sha1\" source property is deprecated due to the weakness of SHA1 hashes.\n");
+          g_printerr ("Use the \"sha256\" property for the more secure SHA256 hash.\n");
+        }
+
       break;
 
     case PROP_SHA256:
@@ -182,6 +201,11 @@ builder_source_file_set_property (GObject      *object,
       g_strfreev (tmp);
       break;
 
+    case PROP_HTTP_REFERER:
+      g_free (self->http_referer);
+      self->http_referer = g_value_dup_string (value);
+      break;
+
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
     }
@@ -201,11 +225,11 @@ builder_source_file_validate (BuilderSource  *source,
 }
 
 
-static SoupURI *
+static GUri *
 get_uri (BuilderSourceFile *self,
          GError           **error)
 {
-  SoupURI *uri;
+  GUri *uri;
 
   if (self->url == NULL)
     {
@@ -213,10 +237,9 @@ get_uri (BuilderSourceFile *self,
       return NULL;
     }
 
-  uri = soup_uri_new (self->url);
+  uri = g_uri_parse (self->url, CONTEXT_HTTP_URI_FLAGS, error);
   if (uri == NULL)
     {
-      g_set_error (error, G_IO_ERROR, G_IO_ERROR_FAILED, "Invalid URL '%s'", self->url);
       return NULL;
     }
   return uri;
@@ -228,7 +251,7 @@ get_download_location (BuilderSourceFile *self,
                        BuilderContext    *context,
                        GError           **error)
 {
-  g_autoptr(SoupURI) uri = NULL;
+  g_autoptr(GUri) uri = NULL;
   const char *path;
   g_autofree char *base_name = NULL;
   g_autoptr(GFile) file = NULL;
@@ -239,7 +262,7 @@ get_download_location (BuilderSourceFile *self,
   if (uri == NULL)
     return FALSE;
 
-  path = soup_uri_get_path (uri);
+  path = g_uri_get_path (uri);
 
   if (g_str_has_prefix (self->url, "data:"))
     {
@@ -317,23 +340,27 @@ download_data_uri (const char     *url,
                    BuilderContext *context,
                    GError        **error)
 {
-  SoupSession *session;
-
-  g_autoptr(SoupRequest) req = NULL;
+  CURL *session;
+  g_autoptr(GError) first_error = NULL;  
   g_autoptr(GInputStream) input = NULL;
   g_autoptr(GOutputStream) out = NULL;
+  g_autoptr(GUri) parsed = NULL;
 
-  session = builder_context_get_soup_session (context);
-
-  req = soup_session_request (session, url, error);
-  if (req == NULL)
-    return NULL;
-
-  input = soup_request_send (req, NULL, error);
-  if (input == NULL)
-    return NULL;
+  parsed = g_uri_parse(url, CONTEXT_HTTP_URI_FLAGS, error);
+  if (!parsed)
+    {
+      return FALSE;
+    }
 
+  session = builder_context_get_curl_session (context);
   out = g_memory_output_stream_new_resizable ();
+
+  if (!builder_download_uri_buffer (parsed, NULL, session, out, NULL, 0, error))
+    {
+      g_propagate_error (error, g_steal_pointer (&first_error));
+      return NULL;
+    }
+
   if (!g_output_stream_splice (out,
                                input,
                                G_OUTPUT_STREAM_SPLICE_CLOSE_TARGET | G_OUTPUT_STREAM_SPLICE_CLOSE_SOURCE,
@@ -408,6 +435,7 @@ builder_source_file_download (BuilderSource  *source,
   if (!builder_context_download_uri (context,
                                      self->url,
                                      (const char **)self->mirror_urls,
+                                     self->http_referer,
                                      file,
                                      checksums,
                                      checksums_type,
@@ -671,6 +699,14 @@ builder_source_file_class_init (BuilderSourceFileClass *klass)
                                                        "",
                                                        G_TYPE_STRV,
                                                        G_PARAM_READWRITE));
+
+  g_object_class_install_property (object_class,
+                                   PROP_HTTP_REFERER,
+                                   g_param_spec_string ("referer",
+                                                        "",
+                                                        "",
+                                                        NULL,
+                                                        G_PARAM_READWRITE));
 }
 
 static void
